@@ -40,12 +40,6 @@ static void Rhpc_worker_lapply_seq(int *cmd)
   R_xlen_t XLlen;
   R_xlen_t works;
 
-  /* serialize */
-  SEXP l_out,out;
-
-  /* result list */
-  SEXP resL;
-  PROTECT_INDEX resL_ix;
 
   GET_CMD(cmd, &getcmd, &getsubcmd, &cnti, &modi);
 
@@ -176,8 +170,6 @@ static void Rhpc_worker_lapply_seq(int *cmd)
   }
 
   XLlen = xlength(XL);
-  PROTECT(resL = allocVector(VECSXP,XLlen));
-  PROTECT_WITH_INDEX(resL,&resL_ix);
   for(works=0; works<XLlen; works ++){
     SEXP X;
     /* make args */
@@ -187,6 +179,8 @@ static void Rhpc_worker_lapply_seq(int *cmd)
     R_xlen_t argw_len;
     /* eval */
     SEXP lng,ret;
+    /* serialize */
+    SEXP l_out,out;
 
     PROTECT(X   = VECTOR_ELT(XL,works));
 
@@ -231,89 +225,84 @@ static void Rhpc_worker_lapply_seq(int *cmd)
       setAttrib(elist, R_ClassSymbol, eclass);
       ret=elist;
     }
-    /* PROTECT(ret); */
-    SET_VECTOR_ELT(resL,works,ret);
-    REPROTECT(resL,resL_ix);
-    UNPROTECT(5);
-  }
-
-  DPRINT("success eval x\n");
-
-  /* serialize */
-  {
+    PROTECT(ret);
+    DPRINT("success eval x\n");
 
+    /* serialize */
     if(SERMODE){
-      l_out = LCONS(install("serialize"),CONS(resL,CONS(R_NilValue, CONS(ScalarLogical(FALSE), CONS(R_NilValue, CONS(R_NilValue, R_NilValue))))));
+      l_out = LCONS(install("serialize"),CONS(ret,CONS(R_NilValue, CONS(ScalarLogical(FALSE), CONS(R_NilValue, CONS(R_NilValue, R_NilValue))))));
       l_out = LCONS(install(".Internal"), CONS(l_out, R_NilValue));
       PROTECT(l_out);
       PROTECT(out=R_tryEval(l_out, R_GlobalEnv, &errorOccurred));
     }else{/* SERMODE */
       PROTECT(l_out=R_NilValue);
-      PROTECT(out=Rhpc_serialize(resL));
+      PROTECT(out=Rhpc_serialize(ret));
     }
     DPRINT("success  serialize outlength=%ld\n",xlength(out));
-  }
 
-  /* send */
-  {
-    int cmdo[CMDLINESZ];
 
-    R_xlen_t sz =  xlength(out);
-    R_xlen_t cnto = (int)(sz / RHPC_SPLIT_SIZE);
-    R_xlen_t modo = (int)(sz % RHPC_SPLIT_SIZE);
-    int reqcnt = cnto+((modo)?1:0);
+    /* send */
+    {
+      int cmdo[CMDLINESZ];
 
-    MPI_Request *request = Calloc(reqcnt+1, MPI_Request);
-    MPI_Status  *status  = Calloc(reqcnt+1, MPI_Status);
-    int calls;
+      R_xlen_t sz =  xlength(out);
+      R_xlen_t cnto = (int)(sz / RHPC_SPLIT_SIZE);
+      R_xlen_t modo = (int)(sz % RHPC_SPLIT_SIZE);
+      int reqcnt = cnto+((modo)?1:0);
 
-    SET_CMD(cmdo, CMD_NAME_LAPPLY_SEQ, SUBCMD_NORMAL, cnto, modo );
+      MPI_Request *request = Calloc(reqcnt+1, MPI_Request);
+      MPI_Status  *status  = Calloc(reqcnt+1, MPI_Status);
+      int calls;
 
-    calls=0;
-    if(SYNC){
-      _M(MPI_Send(cmdo,   (int)CMDLINESZ,
-		  MPI_INT, 0, RHPC_CTRL_TAG,
-		  RHPC_Comm));
-    }else{
-      _M(MPI_Isend(cmdo,   (int)CMDLINESZ,
-		   MPI_INT, 0, RHPC_CTRL_TAG,
-		   RHPC_Comm, &request[calls]));
-    }
-    DPRINT("send ctrl\n");
-    calls++;
-    for( i = 0 ; i< cnto ; i++){
+      SET_CMD(cmdo, CMD_NAME_LAPPLY_SEQ, SUBCMD_NORMAL, cnto, modo );
+
+      calls=0;
       if(SYNC){
-	_M(MPI_Send(RAW(out)+RHPC_SPLIT_SIZE*i,   (int)RHPC_SPLIT_SIZE,
-		    MPI_CHAR, 0, TAGCAL(i),
+	_M(MPI_Send(cmdo,   (int)CMDLINESZ,
+		    MPI_INT, 0, RHPC_CTRL_TAG,
 		    RHPC_Comm));
       }else{
-	_M(MPI_Isend(RAW(out)+RHPC_SPLIT_SIZE*i,   (int)RHPC_SPLIT_SIZE,
-		     MPI_CHAR, 0, TAGCAL(i),
+	_M(MPI_Isend(cmdo,   (int)CMDLINESZ,
+		     MPI_INT, 0, RHPC_CTRL_TAG,
 		     RHPC_Comm, &request[calls]));
       }
+      DPRINT("send ctrl\n");
       calls++;
-    }
-    if ( modo != 0 ){
-      if(SYNC){
-	_M(MPI_Send(RAW(out)+RHPC_SPLIT_SIZE*cnto, (int)modo,
-		    MPI_CHAR, 0, TAGCAL(cnto),
-		    RHPC_Comm));
-      }else{
-	_M(MPI_Isend(RAW(out)+RHPC_SPLIT_SIZE*cnto, (int)modo,
-		     MPI_CHAR, 0, TAGCAL(cnto),
-		     RHPC_Comm, &request[calls]));
+      for( i = 0 ; i< cnto ; i++){
+	if(SYNC){
+	  _M(MPI_Send(RAW(out)+RHPC_SPLIT_SIZE*i,   (int)RHPC_SPLIT_SIZE,
+		       MPI_CHAR, 0, TAGCAL(i),
+		       RHPC_Comm));
+	}else{
+	  _M(MPI_Isend(RAW(out)+RHPC_SPLIT_SIZE*i,   (int)RHPC_SPLIT_SIZE,
+		       MPI_CHAR, 0, TAGCAL(i),
+		       RHPC_Comm, &request[calls]));
+	}
+	calls++;
       }
-      calls++;
+      if ( modo != 0 ){
+	if(SYNC){
+	  _M(MPI_Send(RAW(out)+RHPC_SPLIT_SIZE*cnto, (int)modo,
+		       MPI_CHAR, 0, TAGCAL(cnto),
+		       RHPC_Comm));
+	}else{
+	  _M(MPI_Isend(RAW(out)+RHPC_SPLIT_SIZE*cnto, (int)modo,
+		       MPI_CHAR, 0, TAGCAL(cnto),
+		       RHPC_Comm, &request[calls]));
+	}
+	calls++;
+      }
+      DPRINT("send data wait start\n");
+      if(!SYNC){
+	_M(MPI_Waitall(calls, request, status));
+      }
+      Free(request);
+      Free(status);
+      DPRINT("send data wait end\n");
     }
-    DPRINT("send data wait start\n");
-    if(!SYNC){
-      _M(MPI_Waitall(calls, request, status));
-    }
-    Free(request);
-    Free(status);
-    DPRINT("send data wait end\n");
+    UNPROTECT(8);
   }
-  UNPROTECT(11);
+  UNPROTECT(9);
 
   /* recv exit */
   {
